import datetime
import random
from footballtestdata.data import ENG, ITA, SPA, GER, position

class FootballerGenerator:
    def __init__(self, first_names, last_names, cities, club, position):
        self.first_names = first_names
        self.last_names = last_names
        self.cities = cities
        self.club = club
        self.position = position
        
    @staticmethod
    def generate_random_date_of_birth(start_year, end_year):
        start_date = datetime.date(start_year, 1, 1)
        end_date = datetime.date(end_year, 12, 31)
        random_date = start_date + datetime.timedelta(days=random.randint(0, (end_date - start_date).days))
        return random_date

    def generate_fake_footballer(self, nationality):
        first_name = random.choice(self.first_names)
        last_name = random.choice(self.last_names)
        club = random.choice(self.cities) + ' ' + random.choice(self.club)
        position = random.choice(self.position)
        date_of_birth = self.generate_random_date_of_birth(1986, 2005)
        matches_played = random.randint(1, 34)
        minutes_played = random.randint(matches_played, matches_played * 90)
        # TODO Make goals and assists more realistic
        # TODO Add more statistics: xG and others
        goals = (lambda pos: 0 if pos == "GK" else random.randint(0, min(minutes_played // 50, 6)) if pos in ["LB", "CB", "RB"] else random.randint(0, min(minutes_played // 50, 9)) if pos in ["DM", "CM"] else random.randint(0, min(minutes_played // 50, 12)) if pos in ["AM", "LM", "RM"] else random.randint(0, min(minutes_played // 30, 28)))(position)
        assists = (lambda pos: random.randint(0, min(minutes_played // 50, 3)) if pos == "GK" else random.randint(0, min(minutes_played // 50, 6)) if pos in ["LB", "CB", "RB"] else random.randint(0, min(minutes_played // 50, 9)) if pos in ["DM", "CM"] else random.randint(0, min(minutes_played // 50, 12)) if pos in ["AM", "LM", "RM"] else random.randint(0, min(minutes_played // 50, 19)))(position)


        return {
            'first_name': first_name,
            'last_name': last_name,
            'position': position,
            'nationality': nationality,
            'club': club,
            'dob': date_of_birth, 
            'matches_played': matches_played,
            'minutes_played': minutes_played,
            'goals': goals,
            'assists': assists
        }
    # TODO add a piece of code that allows you to create one fake club with 23 players
    


# There are 4 lists: England (ENG), Italy (ITA), Spain (SPA), Germany (GER)

ENG = [
    # English male names
    ["John", "Michael", "William", "James", "David", "Robert", "Joseph", "Daniel", "Matthew", "Andrew",
     "Richard", "Thomas", "Charles", "Anthony", "Christopher", "Mark", "Paul", "Steven", "Kevin", "Brian",
     "George", "Edward", "Jason", "Timothy", "Ryan", "Stephen", "Scott", "Kenneth", "Nathan", "Donald",
     "Jeffrey", "Eric", "Jacob", "Patrick", "Brandon", "Gary", "Jonathan", "Dennis", "Frank", "Gregory",
     "Benjamin", "Jerry", "Tyler", "Samuel", "Raymond", "Adam", "Peter", "Henry", "Walter", "Keith"],

    # English last names
    ["Smith", "Johnson", "Brown", "Taylor", "Miller", "Anderson", "Wilson", "Clark", "White", "Walker",
     "Hall", "Young", "Allen", "King", "Wright", "Lewis", "Hill", "Scott", "Green", "Adams", "Baker",
     "Carter", "Turner", "Parker", "Collins", "Morris", "Cook", "Cooper", "Morgan", "Murphy", "Bailey",
     "Kelly", "Howard", "Hughes", "Russell", "Cruz", "Brooks", "Long", "Butler", "Foster", "Gonzalez",
     "Bryant", "Graham", "Sanders", "Reed", "Diaz", "Sullivan", "Barnes", "Hayes", "Lopez"],

    # English cities
    ["London", "Manchester", "Birmingham", "Liverpool", "Leeds", "Newcastle", "Sheffield", "Bristol", "Oxford",
    "Cambridge", "Nottingham", "Southampton", "York", "Leicester", "Brighton", "Portsmouth", "Norwich", "Exeter",
    "Coventry", "Derby", "Reading", "Swansea", "Milton Keynes", "Stoke-on-Trent", "Plymouth", "Bradford",
    "Wolverhampton", "Sunderland", "Luton", "Newport", "Preston", "Swindon", "Huddersfield", "Middlesbrough",
    "Blackpool", "Bournemouth", "Stockport", "Peterborough", "Warrington", "Slough", "Colchester", "Gloucester",
    "Watford", "Ipswich", "Northampton", "Bath", "Chester", "Salford", "Carlisle", "Worcester", "Chelmsford",
    "Doncaster", "Birkenhead", "Guildford", "Aylesbury", "Basingstoke", "High Wycombe", "Burnley", "Basildon",
    "Crawley", "Woking", "Lincoln", "Gillingham", "Wigan", "Oldham", "Rochdale", "Sutton Coldfield", "Solihull",
    "Eastbourne", "Runcorn", "Grimsby", "Barnsley", "Ipswich", "Poole", "Telford", "York", "Blackburn",
    "Mansfield", "Darlington", "Gravesend", "Dewsbury", "Blyth", "Exmouth", "Brentwood", "Batley", "Chorley",
    "Scunthorpe", "Halesowen", "St Albans", "Lowestoft", "Canterbury", "Ellesmere Port", "Farnborough",
    "Beeston", "Harrogate", "Chesterfield", "Great Yarmouth", "Barrow-in-Furness", "Folkestone", "Grantham",
    "Sittingbourne", "Potters Bar", "Redditch", "Burgess Hill", "Rochester", "Banbury", "St Helens", "Kettering",
    "Widnes", "Kidderminster", "Bedford", "Stourbridge", "Tamworth", "Keighley", "Stroud", "Fareham", "Hereford",
    "Hemel Hempstead", "Wellingborough", "Loughborough", "Bury", "Bishop's Stortford", "Filton", "Bromsgrove",
    "Hatfield", "Chippenham", "Aldershot", "Newton Abbot", "Gosport", "Corby", "Staines", "Letchworth",
    "Trowbridge", "Hitchin", "Frome", "Farnham", "Yate", "Ashford", "Canvey Island", "Glossop", "Deal",
    "Dartford", "Dunstable", "Swadlincote", "Horsham", "Wisbech", "Cramlington", "Redcar", "Hamilton"],

    # English club names
    ["Fc", "City", "United", "Athletic", "County", "Rangers", "Rovers"]
]

ITA = [
    # Italian male names
    ["Alessandro", "Andrea", "Antonio", "Armando", "Bruno", "Carlo", "Daniele", "Dario", "Diego", "Edoardo", "Enrico", "Fabio", "Federico", "Filippo", 
     "Francesco", "Gabriele", "Giacomo", "Giampaolo", "Giampiero", "Giancarlo", "Gianfranco", "Gianluca", "Gianmarco", "Gianni", "Giovanni", "Giuliano", 
     "Giulio", "Guido", "Ignazio", "Ivan", "Lorenzo", "Luca", "Luciano", "Marco", "Mario", "Massimo", "Matteo", "Maurizio", "Mauro", "Michele", "Mirko", 
     "Nicola", "Paolo", "Pierluigi", "Piero", "Pietro", "Riccardo", "Roberto", "Salvatore", "Samuele", "Sergio", "Simone", "Stefano", "Tommaso", "Umberto", 
     "Valerio", "Vincenzo", "Walter", "Alberto", "Angelo", "Aurelio", "Cesare", "Cristiano", "Davide", "Emanuele", "Fausto", "Fernando", "Giovanni", 
     "Guglielmo", "Igor", "Leonardo", "Manuel", "Marcello", "Marco", "Matteo", "Maurizio", "Nicolò", "Pietro", "Roberto", "Silvano", "Vittorio", "Alberto", 
     "Alessio", "Andrea", "Antonio", "Benedetto", "Cesare", "Cristiano", "Daniele", "Davide", "Diego", "Edoardo", "Enrico", "Fabio", "Fabrizio"],

    # Italian last names
    ["Rossi", "Russo", "Ferrari", "Esposito", "Bianchi", "Romano", "Colombo", "Ricci", "Marino", "Greco", "Bruno", "Gallo", "Conti", "De Luca", "Mancini",
    "Costa", "Giordano", "Rizzo", "Lombardi", "Moretti", "Barbieri", "Fontana", "Santoro", "Mariani", "Rinaldi", "Caruso", "Ferrara", "Galli", "Martini",
    "Leone", "Longo", "Gentile", "Martinelli", "Vitale", "Lombardo", "Serra", "Coppola", "De Santis", "Riva", "De Angelis", "Marchetti", "Parisi",
    "Villa", "Conte", "Ferri", "Bellini", "Bianco", "Rossetti", "D'Amico", "Fabbri", "Vitali", "Sala", "Marini", "Amato", "Monti", "De Rosa",
    "Giuliani", "Gatti", "Sartori", "Neri", "Costantini", "Rizzi", "Farina", "Testa", "Pellegrini", "Bianchini", "Piras", "Longo", "Pagano",
    "Rosso", "Guerra", "Fabbro", "Palumbo", "Salvatore", "Mazza", "Grasso", "Piazza", "Ruggeri", "Lombardi", "Benedetti", "Parisi", "Bellini",
    "Galli", "Mancini", "Valentini", "Sanna", "Cattaneo", "Negri", "Barbieri", "Marini", "D'Angelo", "Caputo", "Montanari", "Coppola", "Basile",
    "D'Amico", "Silvestri", "Pellegrino", "Damico", "Ricci", "Ferrero"],

    # Italian cities
    ["Rome", "Milan", "Naples", "Turin", "Palermo", "Genoa", "Bologna", "Florence", "Bari", "Catania","Venice", "Verona", "Messina", "Padua", "Trieste", 
    "Brescia", "Taranto", "Prato", "Reggio Calabria","Modena", "Parma", "Reggio Emilia", "Perugia", "Livorno", "Ravenna", "Cagliari", "Foggia", "Rimini",
    "Salerno", "Ferrara", "Sassari", "Latina", "Giugliano in Campania", "Monza", "Siracusa", "Pescara", "Bergamo", "Forlì", "Trento", "Vicenza", "Terni", "Bolzano", "Novara", "Piacenza", "Ancona", "Andria",
    "Udine", "Arezzo", "Cesena", "Lecce", "Barletta", "Alessandria", "Pesaro", "La Spezia", "Pisa", "Pistoia",
    "Guidonia Montecelio", "Lucca", "Catanzaro", "Brindisi", "Torre del Greco", "Treviso", "Busto Arsizio",
    "Como", "Marsala", "Grosseto", "Sesto San Giovanni", "Varese", "Fiumicino", "Asti", "Casoria", "Cinisello Balsamo",
    "Gela", "April", "Ragusa", "Pavia", "Cremona", "Carpi", "Quartu Sant'Elena", "Lamezia Terme", "Altamura",
    "Imola", "L\u2019Aquila", "Massa", "Trapani", "Viterbo", "Cosenza", "Potenza", "Castellammare di Stabia", "Afragola",
    "Vittoria", "Crotone", "Pomezia", "Carrara", "Caserta", "Savona", "Molfetta", "Caltanissetta", "Bisceglie",
    "Bitonto", "Siena", "Vercelli", "Vigevano", "Matera", "Olbia", "Benevento", "Agrigento", "Acerra", "Marano di Napoli",
    "Cerignola", "Scafati", "Bagheria", "Sanremo", "Avellino", "Ercolano", "Pordenone", "Battipaglia", "Cava de' Tirreni",
    "Mazara del Vallo", "Velletri", "Scafati", "Molfetta", "Fano", "Rho", "Pomezia", "Acireale", "Modica", "Aprilia",
    "Scandicci", "Carpi", "Caltagirone", "Civitavecchia", "Monserrato", "Cassino", "Corigliano-Rossano", "Bassano del Grappa",
    "Feltre", "Cervia", "Portici", "Gioia del Colle", "San Giorgio a Cremano", "San Giovanni la Punta", "Aci Catena", "Acquaviva delle Fonti",
    "Cantù", "Cinisello Balsamo", "Civitanova Marche", "Fabriano", "Nocera Inferiore", "Cercola", "Giugliano in Campania", "Mirano",
    "Villorba", "Suzzara", "Osimo", "Mugnano di Napoli", "San Giovanni Lupatoto", "Cisterna di Latina", "Foligno", "Senigallia",
    "Terlizzi", "Sassuolo", "Avola", "Torre Annunziata", "Carate Brianza", "Marino", "Arona", "Villafranca di Verona", "Sant'Anastasia",
    "Montecchio Maggiore", "Giarre", "Bassano del Grappa", "Porto Torres", "Carrara", "Carpi", "Biella", "Fasano", "Patti"],

    # Italian club names
    ["Calcio", "FC", "AC"]
] 
SPA = [
    # Spanish first names
    ["Alejandro", "Carlos", "David", "Fernando", "Gabriel", "Hugo", "Iván", "Javier", "Luis", "Manuel",
    "Nicolás", "Óscar", "Pablo", "Raúl", "Santiago", "Tomás", "Víctor", "Xavier", "Yago", "Zacarías",
    "Adrián", "Borja", "César", "Diego", "Emilio", "Federico", "Gonzalo", "Héctor", "Ignacio", "Joaquín",
    "Kevin", "Lorenzo", "Mario", "Nelson", "Óliver", "Pedro", "Quique", "Rubén", "Samuel", "Teo",
    "Ulises", "Vicente", "Walter", "Xavi", "Yeray", "Zaid", "Abel", "Bruno", "Ciro", "Damián",
    "Eduardo", "Felipe", "Gael", "Hugo", "Iker", "Julián", "Kevin", "Leandro", "Mateo", "Nicolás",
    "Óliver", "Pablo", "Quim", "Raúl", "Sergio", "Thiago", "Uriel", "Víctor", "Waldo", "Xavier",
    "Yago", "Zacarías", "Adrián", "Bruno", "César", "Diego", "Eduardo", "Fernando", "Gonzalo", "Hugo",
    "Ignacio", "Javier", "Kevin", "Lorenzo", "Mario", "Nicolás", "Óscar", "Pablo", "Raúl", "Santiago",
    "Tomás", "Víctor", "Xavier", "Yago", "Zacarías"],

    # Spanish last names
    ["García", "Rodríguez", "González", "Fernández", "López", "Martínez", "Pérez", "Sánchez", "Ramírez", "Torres",
    "Flores", "Rivera", "Gómez", "Díaz", "Hernández", "Jiménez", "Morales", "Vargas", "Ramos", "Cruz",
    "Romero", "Ortega", "Reyes", "Mendoza", "Castillo", "Soto", "Delgado", "Cortés", "Vega", "Guerrero",
    "Molina", "Santos", "Chávez", "Medina", "Méndez", "Pardo", "Cabrera", "Acosta", "Campos", "Ríos",
    "Velasco", "Silva", "Pacheco", "Vargas", "Pineda", "Montoya", "Navarro", "Aguilar", "Escobar", "Lara",
    "Suárez", "Miranda", "Serrano", "Zamora", "Iglesias", "Cano", "Rojas", "Guzmán", "Valencia", "Carvajal",
    "Maldonado", "Mora", "Duarte", "Vega", "León", "Zúñiga", "Vera", "Villalobos", "Bermúdez", "Arias",
    "Solís", "Andrade", "Navas", "Morera", "Fuentes", "Salazar", "Barrantes", "Monge", "Monge", "Solano",
    "Ulate", "Benavides", "Lizano", "Sáenz", "Quirós", "Cordero", "Jiménez", "Zúñiga", "Cascante", "Murillo",
    "Calderón", "Vargas", "Brenes", "Marín", "Zúñiga", "Villegas", "Herrera", "Camacho", "Gutiérrez", "Sánchez"],

    # Spanish cities
    ["Madrid", "Barcelona", "Valencia", "Sevilla", "Zaragoza", "Málaga", "Murcia", "Palma", "Bilbao", "Alicante",
    "Córdoba", "Valladolid", "Vigo", "Gijón", "L'Hospitalet de Llobregat", "A Coruña", "Vitoria", "Granada",
    "Elche", "Oviedo", "Badalona", "Terrassa", "Cartagena", "Jerez de la Frontera", "Sabadell", "Móstoles",
    "Santa Cruz de Tenerife", "Alcalá de Henares", "Pamplona", "Fuenlabrada", "Almería", "Leganés", "San Sebastián",
    "Burgos", "Santander", "Castellón de la Plana", "Albacete", "Getafe", "Alcorcón", "San Cristóbal de La Laguna",
    "Logroño", "Badajoz", "Salamanca", "Huelva", "Marbella", "Lleida", "Tarragona", "Cádiz", "León", "Jaén",
    "Ourense", "Algeciras", "Cáceres", "Dos Hermanas", "Reus", "Torrejón de Ardoz", "Parla", "Mataró", "Sant Cugat del Vallès",
    "Alcobendas", "Santiago de Compostela", "Coslada", "Las Palmas de Gran Canaria", "Rubí", "Estepona", "Pontevedra",
    "Chiclana de la Frontera", "Majadahonda", "San Fernando", "Manresa", "Fuengirola", "Benidorm", "El Ejido",
    "Vélez-Málaga", "Mollet del Vallès", "Pozuelo de Alarcón", "Avilés", "Cornellà de Llobregat", "Ferrol", "Arrecife",
    "Orihuela", "Gandia", "Ávila", "Vila-real", "Irun", "Mijas", "Benalmádena", "Motril", "Paterna", "Cártama",
    "Sant Boi de Llobregat", "Rubí", "La Línea de la Concepción", "Roquetas de Mar", "Olot", "Cerdanyola del Vallès",
    "Sant Feliu de Llobregat", "Elda", "Guadalajara", "Santurtzi", "Portugalete"],

    # Spanish club names
    ["Real", "FC", "Athletic", "CF", "AD", "Sporting" ]
] 
GER = [
    # German first names
    ["Max", "Paul", "Lukas", "Ben", "Felix", "Tim", "Leon", "Liam", "Jonas", "Noah", "Julian", "Elias", "David", "Luis", "Moritz", "Jakob", "Simon", "Jan", "Oskar", "Niklas",
    "Philipp", "Tom", "Anton", "Henry", "Emil", "Johannes", "Fabian", "Theo", "Leo", "Adam", "Raphael", "Samuel", "Hannes", "Erik", "Marvin", "Henrik", "Mats", "Jonathan", "Vincent", 
    "Tobias", "Nico", "Benjamin", "Finn", "Lennard", "Alexander", "Kai", "Dominik", "Mathias", "Nils", "Paul", "Marcel", "Simon", "Malte", "Markus", "Oliver", "Rico", "Sebastian", 
    "Daniel", "Christopher", "Manuel", "Arne", "Bastian", "Roman", "Karl", "Timo", "Robert", "Valentin", "Joshua", "Andreas", "Maximilian", "Ferdinand", "Florian", "Jonathan", "Ole", 
    "Leonard", "Fabian", "Luca", "Michael", "Marco", "Patrick", "Dennis", "Stefan", "Christoph", "Marius", "Sven", "Kevin", "Dominik", "Tom", "Johannes", "Lucas", "Rafael", "Maurice", 
    "Sebastian", "Andreas", "Rainer", "Nico", "Henry", "Roman", "Jan", "Tobias"],

    # German last names
    ["Müller", "Schmidt", "Schneider", "Fischer", "Weber", "Meyer", "Wagner", "Becker", "Schulz", "Hoffmann", "Schäfer", "Koch", "Bauer", "Richter", "Klein", "Wolf", "Schröder", "Neumann",
    "Schwarz", "Zimmermann", "Braun", "Krüger", "Hofmann", "Hartmann", "Lange", "Schmitt", "Werner", "Schmitz", "Krause", "Meier", "Lehmann", "Schmid", "Schulze", "Maier", "Köhler", 
    "Herrmann", "Walter", "König", "Mayer", "Huber", "Kaiser", "Fuchs", "Peters", "Lang", "Scholz", "Möller", "Weiß", "Jung", "Hahn", "Schubert", "Vogel", "Friedrich", "Keller", "Günther", 
    "Frank", "Berger", "Winkler", "Roth", "Beck", "Lorenz", "Baumann", "Franke", "Albrecht", "Schuster", "Simon", "Ludwig", "Böhm", "Winter", "Kraus", "Martin", "Krämer", "Vogt", "Stein",
    "Jäger", "Otto", "Sommer", "Groß", "Seidel", "Heinrich", "Brandt", "Schreiber", "Graf", "Schulte", "Dietrich", "Ziegler", "Kuhn", "Kühn", "Pohl", "Engel", "Horn", "Bergmann", "Voigt", 
    "Bayer", "Sauer", "Arndt", "Thomas", "Fiedler", "Renner", "Schütz", "Reuter"],

    # German cities
     ["Berlin", "Hamburg", "München", "Köln", "Frankfurt am Main", "Stuttgart", "Düsseldorf", "Dortmund", "Essen",
    "Leipzig", "Bremen", "Dresden", "Hannover", "Nürnberg", "Duisburg", "Bochum", "Wuppertal", "Bielefeld",
    "Bonn", "Münster", "Karlsruhe", "Mannheim", "Augsburg", "Wiesbaden", "Gelsenkirchen", "Mönchengladbach",
    "Braunschweig", "Chemnitz", "Kiel", "Aachen", "Halle (Saale)", "Magdeburg", "Freiburg im Breisgau", "Krefeld", "Lübeck",
    "Oberhausen", "Erfurt", "Mainz", "Rostock", "Kassel", "Hagen", "Hamm", "Saarbrücken", "Mülheim an der Ruhr",
    "Potsdam", "Ludwigshafen am Rhein", "Oldenburg", "Leverkusen", "Osnabrück", "Solingen", "Heidelberg", "Herne",
    "Neuss", "Darmstadt", "Paderborn", "Regensburg", "Ingolstadt", "Würzburg", "Wolfsburg", "Fürth", "Offenbach am Main",
    "Ulm", "Heilbronn", "Pforzheim", "Göttingen", "Bottrop", "Trier", "Recklinghausen", "Reutlingen",
    "Bremerhaven", "Koblenz", "Bergisch Gladbach", "Jena", "Remscheid", "Erlangen", "Moers", "Siegen",
    "Hildesheim", "Salzgitter", "Cottbus", "Kaiserslautern", "Gütersloh", "Witten", "Hanau", "Schwerin",
    "Gera", "Iserlohn", "Ludwigsburg", "Zwickau", "Esslingen am Neckar", "Düren", "Ratingen", "Flensburg", "Villingen-Schwenningen",
    "Lünen", "Marl", "Konstanz"],

    # German club names
    ["Vfb", "FC", "Borussia", "Vfl", "TsG", "VfR" ]

]


# Player position
position = ["GK", "LB", "CB", "RB", "DM", "CM", "AM", "LM", "RM","ST"]



from setuptools import setup, find_packages

VERSION = '1.0' 
DESCRIPTION = 'A package to generate fake footballer data'
LONG_DESCRIPTION = 'A package to generate fake footballer data and statistics for data science training purpose'

setup(
    name='footballtestdata',
    version='1.0',
    description='A package to generate fake footballer data and statistics',
    author='Marek Zarzycki',
    author_email='contact@mazarzycki.com',
    author_website='https://mazarzycki.com/',
    packages=find_packages(),

    keywords=['python', 'football', 'soccer', 'data'],

)
